/*
Stochastic Theory Education through Visualization Environment (STEVE) 2.0
Robert Florence, Tennessee Tech University
InputForm.cpp - This class creates the input form and animation controls.
Last Modified: July 24, 2008
*/

#include "InputForm.h"

/*
set parent window and create list of parameters
hWndParent: window to create input form in
paramFile: file containing parameter information
*/
InputForm::InputForm(HWND hWndParent, char* paramFile)
{
    int i;
    double min, max, def;
    char name[50];
    std::ifstream inFile(paramFile);
    
    hWnd = hWndParent;
    numParams = 0;
    
    if (!inFile.is_open())
        return;
    
    //get number of parameters in list
    inFile >> numParams;
    params = new Parameter*[numParams];
    
    //ignore column headers
    inFile.ignore(1, '\n');
    inFile.ignore(50, '\n');
    
    //read each parameter from file
    for (i = 0; i < numParams && inFile.good(); i++)
    {
        inFile.getline(name, 50, '\t');
        inFile >> min >> max >> def;
        inFile.ignore(1, '\n');
        
        params[i] = new Parameter(name, min, max, def);
    }
    
    inFile.close();
}

/*
create the input form elements
return: TRUE if all elements are successfully created, otherwise FALSE
*/
BOOL InputForm::createWindows()
{
    int i, nextLine = 20;
    char defaultVal[10];
    BOOL success = TRUE;
    HWND hWndLeftAnim, hWndRightAnim;
    
    //create default title
    if (!CreateWindow(
		"STATIC", //class name
        "Default", //static text
		WS_CHILD | WS_VISIBLE, //window styles
		270, nextLine, //window position (relative)
        60, 20, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_STATIC, //static menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    
    //create custom title
    if (!CreateWindow(
		"STATIC", //class name
        "Custom", //static text
		WS_CHILD | WS_VISIBLE, //window styles
		330, nextLine, //window position (relative)
        60, 20, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_STATIC, //static menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    
    nextLine += 25;
    for (i = 0; i < numParams; i++, nextLine += 30)
    {
        //create parameter title
        if (!CreateWindow(
			"STATIC", //class name
            params[i]->getName(), //static text
			WS_CHILD | WS_VISIBLE, //window styles
			20, nextLine, //window position (relative)
            220, 20, //window size (pixels)
            hWnd, //parent window
            (HMENU) IDC_STATIC, //static menu id
            GetModuleHandle(NULL), //application instance handle
            NULL //window creation data
            ))
        {
            success = FALSE;
        }
        
        //create parameter default value
        if (i < numParams - 1)
            sprintf(defaultVal, "%g", params[i]->getDefault());
        else
            strncpy(defaultVal, "N/A", 4);
            
        if (!CreateWindow(
			"STATIC", //class name
            defaultVal, //static text
			WS_CHILD | WS_VISIBLE, //window styles
			270, nextLine, //window position (relative)
            40, 20, //window size (pixels)
            hWnd, //parent window
            (HMENU) IDC_STATIC, //static menu id
            GetModuleHandle(NULL), //application instance handle
            NULL //window creation data
            ))
        {
            success = FALSE;
        }

        //create parameter input field
        if (!CreateWindow(
			"EDIT", //class name
            "", //edit text
			WS_CHILD | WS_BORDER | WS_VISIBLE | ES_AUTOHSCROLL | WS_TABSTOP, //window styles
			330, nextLine, //window position (relative)
            50, 20, //window size
            hWnd, //parent window
            (HMENU) (IDC_INPUTVAL + i), //unique edit menu id
            GetModuleHandle(NULL), //application instance handle
            NULL //window creation data
            ))
        {
            success = FALSE;
        }
    }

    //create start button
    nextLine += 5;
    if (!CreateWindow(
		"BUTTON", //class name
        "Start", //button text
		WS_CHILD | WS_BORDER | WS_VISIBLE | WS_TABSTOP, //window styles
		300, nextLine, //window position (relative)
        80, 24, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_START, //button menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    
    //create animation control group box
    nextLine += 30;
    if (!CreateWindow(
		"BUTTON", //class name
        "Animation Control", //button text
		WS_CHILD | WS_VISIBLE | BS_GROUPBOX, //window styles
		20, nextLine, //window position (relative)
        360, 60, //window size (pixels)
        hWnd, //parent window
        NULL, //button menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    
    //create back button
    nextLine += 25;
    if (!CreateWindow(
		"BUTTON", //class name
        "", //button text
		WS_CHILD | WS_BORDER | WS_VISIBLE | BS_BITMAP | WS_TABSTOP, //window styles
		40, nextLine, //window position (relative)
        40, 24, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_BACK, //button menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    SendDlgItemMessage(hWnd, IDC_BACK, BM_SETIMAGE, IMAGE_BITMAP, (LPARAM) LoadImage(GetModuleHandle(NULL), "img/back.bmp", IMAGE_BITMAP, 16, 16, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS));
    
    //create play/pause button
    if (!CreateWindow(
		"BUTTON", //class name
        "", //button text
		WS_CHILD | WS_BORDER | WS_VISIBLE | BS_BITMAP | WS_TABSTOP, //window styles
		80, nextLine, //window position (relative)
        40, 24, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_PLAYPAUSE, //button menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    //load play/pause images
    hPlayImg = LoadImage(GetModuleHandle(NULL), "img/play.bmp", IMAGE_BITMAP, 16, 16, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS);
    hPauseImg = LoadImage(GetModuleHandle(NULL), "img/pause.bmp", IMAGE_BITMAP, 16, 16, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS);
    SendDlgItemMessage(hWnd, IDC_PLAYPAUSE, BM_SETIMAGE, IMAGE_BITMAP, (LPARAM) hPauseImg);
    
    //create forward button
    if (!CreateWindow(
		"BUTTON", //class name
        "", //button text
		WS_CHILD | WS_BORDER | WS_VISIBLE | BS_BITMAP | WS_TABSTOP, //window styles
		120, nextLine, //window position (relative)
        40, 24, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_FORWARD, //button menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    SendDlgItemMessage(hWnd, IDC_FORWARD, BM_SETIMAGE, IMAGE_BITMAP, (LPARAM) LoadImage(GetModuleHandle(NULL), "img/forward.bmp", IMAGE_BITMAP, 16, 16, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS));
    
    //create animation speed trackbar
    InitCommonControls();
    if (!CreateWindowEx(
        0, //extended window styles
		TRACKBAR_CLASS, //class name
        "", //trackbar text
		WS_CHILD | WS_VISIBLE | TBS_AUTOTICKS | WS_TABSTOP, //window styles
		210, nextLine, //window position (relative)
        120, 20, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_ANIMSPEED, //trackbar menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    SendDlgItemMessage(hWnd, IDC_ANIMSPEED, TBM_SETRANGE, (WPARAM) TRUE, (LPARAM) MAKELONG(MIN_ANIMSPEED, MAX_ANIMSPEED));

    //create left trackbar label
    if (!(hWndLeftAnim = CreateWindowEx(
        0,
        "STATIC",
        "Slow",
        WS_CHILD | WS_VISIBLE | SS_RIGHT,
        0, 0,
        40, 20,
        hWnd,
        (HMENU) IDC_STATIC,
        GetModuleHandle(NULL),
        NULL
        )))
    {
        success = FALSE;
    }
    SendDlgItemMessage(hWnd, IDC_ANIMSPEED, TBM_SETBUDDY, (WPARAM) TRUE, (LPARAM) hWndLeftAnim);

    //create right trackbar label
    if (!(hWndRightAnim = CreateWindowEx(
        0,
        "STATIC",
        "Fast",
        WS_CHILD | WS_VISIBLE | SS_LEFT,
        0, 0,
        40, 20,
        hWnd,
        (HMENU) IDC_STATIC,
        GetModuleHandle(NULL),
        NULL
        )))
    {
        success = FALSE;
    }
    SendDlgItemMessage(hWnd, IDC_ANIMSPEED, TBM_SETBUDDY, (WPARAM) FALSE, (LPARAM) hWndRightAnim);

    //set default animation speed
    SendDlgItemMessage(hWnd, IDC_ANIMSPEED, TBM_SETPOS, (WPARAM) TRUE, (LPARAM) 7);
    
    //create simulation generation time
    nextLine += 45;
    if (!CreateWindow(
		"STATIC", //class name
        "", //static text
		WS_CHILD | WS_VISIBLE, //window styles
		20, nextLine, //window position (relative)
        360, 20, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_GENTIME, //generation time menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }

    //create TTU logo image
    if (!CreateWindow(
		"STATIC", //class name
        "", //static text
		WS_CHILD | WS_VISIBLE | SS_BITMAP, //window styles
		20, 515, //window position (relative)
        0, 0, //window size (pixels)
        hWnd, //parent window
        (HMENU) IDC_TTULOGO, //generation time menu id
        GetModuleHandle(NULL), //application instance handle
        NULL //window creation data
        ))
    {
        success = FALSE;
    }
    SendDlgItemMessage(hWnd, IDC_TTULOGO, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) LoadImage(GetModuleHandle(NULL), "img/TTUlogo.bmp", IMAGE_BITMAP, 258, 65, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS));

    //set default input values
    reset();
    
    return success;
}

/*
reset all parameters in the input form to their default value
*/
void InputForm::reset()
{
    int i;
    char inputVal[10];
    
    for (i = 0; i < numParams; i++)
    {
        //set input value to parameter's default value
        sprintf(inputVal, "%g", params[i]->getDefault());
        SetDlgItemText(hWnd, IDC_INPUTVAL + i, inputVal);
    }
}

/*
test if all input values are within the valid range for each parameter
return: TRUE if all input values are valid, otherwise FALSE
*/
BOOL InputForm::checkInputs()
{
    int i;
    char inputValStr[10], errorMsg[numParams * 50];
    double inputVal;
    BOOL invalidFound = FALSE;
    
    //create error message
    strcpy(errorMsg, "Invalid Input Values:\r\n");
    
    for (i = 0; i < numParams; i++)
    {
        //get input value
        GetDlgItemText(hWnd, IDC_INPUTVAL + i, inputValStr, 10);
        inputVal = atof(inputValStr);
        
        //check min and max values
        if (inputVal < params[i]->getMin() || inputVal > params[i]->getMax())
        {
            //add parameter to error message
            sprintf(errorMsg, "%s\r\n%s (%g to %g)", errorMsg, params[i]->getName(), params[i]->getMin(), params[i]->getMax());

            invalidFound = TRUE;
        }
    }
    
    //at least one invalid input value
    if (invalidFound)
    {
        MessageBox(NULL, errorMsg, "Error", MB_ICONEXCLAMATION | MB_OK);
        return FALSE;
    }
    
    return TRUE;
}

/*
write all input values to a file, separated by new lines
filename: file to write input values to
*/
void InputForm::writeInputs(char* filename)
{
    int i;
    char inputValStr[10];
    double inputVal;
    std::ofstream outFile(filename);
    
    //write all parameters up to simulation period
    for (i = 0; i < numParams - 1; i++)
    {
        GetDlgItemText(hWnd, IDC_INPUTVAL + i, inputValStr, 10);
        inputVal = atof(inputValStr);
        outFile << inputVal << "\n";
    }
    
    //write simulation period (must be integer)
    GetDlgItemText(hWnd, IDC_INPUTVAL + i, inputValStr, 10);
    inputVal = atof(inputValStr);
    outFile << (int) inputVal << "\n";
    
    outFile.close();
}

/*
read all input values from a file
filename: file to read input values from
*/
void InputForm::readInputs(char* filename)
{
    int i;
    char inputVal[10];
    std::ifstream inFile(filename);

    //set input values to values in file
    for (i = 0; i < numParams && inFile.good(); i++)
    {
        inFile >> inputVal;
        SetDlgItemText(hWnd, IDC_INPUTVAL + i, inputVal);
    }

    inFile.close();
}

/*
prompt for a text file and load settings stored in the file
*/
void InputForm::loadSettings()
{
    OPENFILENAME ofn;
    char filename[MAX_PATH] = "";

    //create open file dialog
    ZeroMemory(&ofn, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = hWnd;
    ofn.lpstrFilter = "Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
    ofn.lpstrFile = filename;
    ofn.nMaxFile = MAX_PATH;
    ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
    ofn.lpstrDefExt = "txt";

    //load input values from specified file
    if(GetOpenFileName(&ofn))
        readInputs(ofn.lpstrFile);
}

/*
prompt for a filename and save settings to that file
*/
void InputForm::saveSettings()
{
    OPENFILENAME ofn;
    char filename[MAX_PATH] = "";

    //create save file dialog
    ZeroMemory(&ofn, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = hWnd;
    ofn.lpstrFilter = "Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
    ofn.lpstrFile = filename;
    ofn.nMaxFile = MAX_PATH;
    ofn.lpstrInitialDir = "";
    ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
    ofn.lpstrDefExt = "txt";

    //save input values to specified file
    if(GetSaveFileName(&ofn))
        writeInputs(ofn.lpstrFile);
}

/*
alternate between play and pause in the animation controls
paused: current state of the play/pause button
return: new state of the play/pause button (!paused)
*/
BOOL InputForm::playPause(BOOL paused)
{
    //unpause, show pause button
    if (paused)
        SendDlgItemMessage(hWnd, IDC_PLAYPAUSE, BM_SETIMAGE, IMAGE_BITMAP, (LPARAM) hPauseImg);
    //pause, show play button
    else
        SendDlgItemMessage(hWnd, IDC_PLAYPAUSE, BM_SETIMAGE, IMAGE_BITMAP, (LPARAM) hPlayImg);
        
    return !paused;
}

/*
get time delay for each frame based on animation speed trackbar
return: seconds until next frame should be drawn (fastest is 0.33)
*/
double InputForm::getAnimDelay()
{
    //get current animation speed trackbar position
    int speed = SendDlgItemMessage(hWnd, IDC_ANIMSPEED, TBM_GETPOS, 0, 0);
    
    //fastest setting is 0.33 seconds, +0.33 for each left position
    return 0.33*(MAX_ANIMSPEED - speed + 1);
}

/*
display the simulation generation time (mm:ss)
seconds: number of seconds it took to run the simulation
*/
void InputForm::dispGenTime(int seconds)
{
    char genTimeStr[50];

    //update generation time text
    sprintf(genTimeStr, "Simulation Generation Time: %02d:%02d", seconds/60, seconds % 60);
    SetDlgItemText(hWnd, IDC_GENTIME, genTimeStr);
}

/*
get number of parameters in the form
return: number of parameters
*/
int InputForm::getNumParams()
{
    return numParams;
}
