/*
Stochastic Theory Education through Visualization Environment (STEVE) 2.0
Robert Florence, Tennessee Tech University
InputForm.h - This class creates the input form and animation controls.
Last Modified: July 24, 2008
*/

#ifndef INPUTFORM_H
#define INPUTFORM_H

#include <windows.h>
#include <commctrl.h>
#include <cstdio>
#include <cstdlib>
#include <fstream>
#include "Parameter.h"
#include "resource.h"

//range of animation speed trackbar
#define MIN_ANIMSPEED 1
#define MAX_ANIMSPEED 10
//ID numbers of input form elements
#define IDC_STATIC -1
#define IDC_INPUTVAL 200
#define IDC_START 300
#define IDC_BACK 301
#define IDC_PLAYPAUSE 302
#define IDC_FORWARD 303
#define IDC_ANIMSPEED 304
#define IDC_GENTIME 400
#define IDC_TTULOGO 500

class InputForm
{
    public:
        /*
        set parent window and create list of parameters
        hWndParent: window to create input form in
        paramFile: file containing parameter information
        */
        InputForm(HWND hWndParent, char* paramFile);
        /*
        create the input form elements
        return: TRUE if all elements are successfully created, otherwise FALSE
        */
        BOOL createWindows();
        /*
        reset all parameters in the input form to their default value
        */
        void reset();
        /*
        test if all input values are within the valid range for each parameter
        return: TRUE if all input values are valid, otherwise FALSE
        */
        BOOL checkInputs();
        /*
        write all input values to a file, separated by new lines
        filename: file to write input values to
        */
        void writeInputs(char* filename);
        /*
        read all input values from a file
        filename: file to read input values from
        */
        void readInputs(char* filename);
        /*
        prompt for a text file and load settings stored in the file
        */
        void loadSettings();
        /*
        prompt for a filename and save settings to that file
        */
        void saveSettings();
        /*
        alternate between play and pause in the animation controls
        paused: current state of the play/pause button
        return: new state of the play/pause button (!paused)
        */
        BOOL playPause(BOOL paused);
        /*
        get time delay for each frame based on animation speed trackbar
        return: seconds until next frame should be drawn (fastest is 0.33)
        */
        double getAnimDelay();
        /*
        display the simulation generation time (mm:ss)
        seconds: number of seconds it took to run the simulation
        */
        void dispGenTime(int seconds);
        /*
        get number of parameters in the form
        return: number of parameters
        */
        int getNumParams();
        
    private:
        //handle to window input form is created in
        HWND hWnd;
        //handles to images for the play/pause button
        HGDIOBJ hPlayImg;
        HGDIOBJ hPauseImg;
        //number of parameters in input form
        int numParams;
        //list of parameters
        Parameter** params;
};

#endif
